@@ -48,12 +48,15 @@ class KateDocumentInfo
 {
   public:
     KateDocumentInfo ()
-        : modifiedOnDisc (false),
-        modifiedOnDiscReason (KTextEditor::ModificationInterface::OnDiskUnmodified)
+        : modifiedOnDisc (false)
+        , modifiedOnDiscReason (KTextEditor::ModificationInterface::OnDiskUnmodified)
+        , openedByUser(false)
     {}
 
     bool modifiedOnDisc;
     KTextEditor::ModificationInterface::ModifiedOnDiskReason modifiedOnDiscReason;
+
+    bool openedByUser;
 };
 
 class KateDocManager : public QStandardItemModel
@@ -85,12 +88,13 @@ class KateDocManager : public QStandardItemModel
       return m_editor;
     }
 
-    KTextEditor::Document *createDoc ();
+    KTextEditor::Document *createDoc (const KateDocumentInfo& docInfo = KateDocumentInfo());
+    
     void deleteDoc (KTextEditor::Document *doc);
 
     KTextEditor::Document *document (uint n);
 
-    const KateDocumentInfo *documentInfo (KTextEditor::Document *doc);
+    KateDocumentInfo *documentInfo (KTextEditor::Document *doc);
 
     int findDocument (KTextEditor::Document *doc);
     /** Returns the documentNumber of the doc with url URL or -1 if no such doc is found */
@@ -109,7 +113,10 @@ class KateDocManager : public QStandardItemModel
       return m_docList;
     }
 
-    KTextEditor::Document *openUrl(const KUrl&, const QString &encoding = QString(), bool isTempFile = false);
+    KTextEditor::Document *openUrl(const KUrl&,
+                                   const QString &encoding = QString(),
+                                   bool isTempFile = false,
+                                   const KateDocumentInfo& docInfo = KateDocumentInfo());
 
     bool closeDocument(class KTextEditor::Document *, bool closeUrl = true);
     bool closeDocument(uint);
@@ -150,16 +157,22 @@ class KateDocManager : public QStandardItemModel
      */
     void saveAll();
 
-	/**
-	 * reloads all documents that has at least one view.
-	 * documents with no views are ignored :P
-	 */
-	void reloadAll();
+    /**
+     * reloads all documents that has at least one view.
+     * documents with no views are ignored :P
+     */
+    void reloadAll();
+
+    /**
+     * close all documents, which could not be reopened
+     */
+    void closeOrphaned();
+
+    /**
+     * save selected documents from the File List
+     */
+    void saveSelected(const QList<KTextEditor::Document*>&);
 
-        /**
-         * close all documents, which could not be reopened
-         */
-        void closeOrphaned();
   Q_SIGNALS:
 
     /**
@@ -175,7 +188,10 @@ class KateDocManager : public QStandardItemModel
     void documentWillBeDeleted (KTextEditor::Document *document);
 
     /**
-     * This signal is emitted when the \p document was deleted.
+     * This signal is emitted when the \p document has been deleted.
+     *    
+     *  Warning !!! DO NOT ACCESS THE DATA REFERENCED BY THE POINTER, IT IS ALREADY INVALID
+     *  Use the pointer only to remove mappings in hash or maps
      */
     void documentDeleted (KTextEditor::Document *document);
 
