@@ -173,7 +173,7 @@ void KateViewManager::slotDocumentOpen ()
   if (cv)
   {
     KEncodingFileDialog::Result r = KEncodingFileDialog::getOpenUrlsAndEncoding(
-                                      cv->document()->encoding(),
+                                      KateDocManager::self()->editor()->defaultEncoding(),
                                       cv->document()->url().url(),
                                       QString(), m_mainWindow, i18n("Open File"));
 
@@ -296,7 +296,7 @@ bool KateViewManager::createView ( KTextEditor::Document *doc )
   delete view->actionCollection()->action( "set_confdlg" );
   delete view->actionCollection()->action( "editor_options" );
 
-  view->setContextMenu(view->defaultContextMenu());
+  //view->setContextMenu(view->defaultContextMenu());
 
   connect(view, SIGNAL(dropEventPass(QDropEvent *)), mainWindow(), SLOT(slotDropEvent(QDropEvent *)));
   connect(view, SIGNAL(focusIn(KTextEditor::View *)), this, SLOT(activateSpace(KTextEditor::View *)));
@@ -647,8 +647,8 @@ void KateViewManager::removeViewSpace (KateViewSpace *viewspace)
       // save current splitter size, as the removal of currentSplitter looses the info
       QList<int> parentSizes = parentSplitter->sizes();
       parentSplitter->insertWidget (index, currentSplitter->widget (0));
-      currentSplitter->hide(); // see bug #232140
-      delete currentSplitter;
+      currentSplitter->hide();
+      delete currentSplitter; // do not delete imidiatelly
       // now restore the sizes again
       parentSplitter->setSizes(parentSizes);
     }
