@@ -8,6 +8,8 @@
   email				: daniel.naber@t-online.de
 
   Copyright (C) 2005 by Anders Lund <anders@alweb.dk>
+
+  KDE SC 4 version (C) 2010 Tomas Trnka <tomastrnka@gmx.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -35,19 +37,9 @@ Also see the user documentation. If backspace is pressed after a completion popu
 was closed, the popup will re-open. This way typos can be corrected and the popup
 will reappear, which is quite comfortable.
 
-FIXME for jowenn if he has time:
--Ctrl-Z doesn't work if completion is visible
--Typing with popup works, but right/left cursor keys and start/end don't, i.e.
- they should be ignored by the completion ( ? )
--popup not completely visible if it's long and appears at the bottom of the screen
-
 FIXME:
 -( docbook ) <author lang="">: insert space between the quotes, press "de" and return -> only "d" inserted
--Correctly support more than one view:
- charactersInteractivelyInserted( ..) is tied to kv->document()
- but filterInsertString( .. ) is tied to kv
 -The "Insert Element" dialog isn't case insensitive, but it should be
--fix upper/lower case problems ( start typing lowercase if the tag etc. is upper case )
 -See the "fixme"'s in the code
 
 TODO:
@@ -82,25 +74,24 @@ TODO:
 #include <qdom.h>
 #include <qfile.h>
 #include <qlayout.h>
-#include <q3listbox.h>
-#include <q3progressdialog.h>
 #include <qpushbutton.h>
 #include <qregexp.h>
 #include <qstring.h>
 #include <qtimer.h>
-//Added by qt3to4:
 #include <QLabel>
 #include <QVBoxLayout>
-#include <Q3ValueList>
 
 #include <kaction.h>
+#include <kactioncollection.h>
+#include <kapplication.h>
 #include <klineedit.h>
-#include <kcursor.h>
 #include <kdebug.h>
 #include <kfiledialog.h>
 #include <kglobal.h>
+#include <khistorycombobox.h>
 #include <kcomponentdata.h>
 #include <kio/job.h>
+#include <kio/jobuidelegate.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kstandarddirs.h>
@@ -108,19 +99,61 @@ TODO:
 
 K_EXPORT_COMPONENT_FACTORY( katexmltoolsplugin, KGenericFactory<PluginKateXMLTools>( "katexmltools" ) )
 
-class PluginView : public KXMLGUIClient
-{
-  friend class PluginKateXMLTools;
+using Kate::application;
 
-  public:
-    Kate::MainWindow *win;
-};
 
 PluginKateXMLTools::PluginKateXMLTools( QObject* parent, const QStringList& )
-  : Kate::Plugin ( (Kate::Application*)parent, "PluginKateXMLTools" )
+    : Kate::Plugin ( (Kate::Application *)parent )
+{
+}
+
+PluginKateXMLTools::~PluginKateXMLTools()
+{
+}
+
+Kate::PluginView *PluginKateXMLTools::createView(Kate::MainWindow *mainWindow)
+{
+    return new PluginKateXMLToolsView(mainWindow);
+}
+
+
+PluginKateXMLToolsView::PluginKateXMLToolsView(Kate::MainWindow *win)
+  : Kate::PluginView ( win ), Kate::XMLGUIClient ( KGenericFactory<PluginKateXMLTools>::componentData() ),
+    m_model ( this )
 {
   //kDebug() << "PluginKateXMLTools constructor called";
 
+
+  KAction *actionInsert = new KAction ( i18n("&Insert Element..."), this );
+  actionInsert->setShortcut( Qt::CTRL+Qt::Key_Return );
+  connect( actionInsert, SIGNAL(triggered()), &m_model, SLOT(slotInsertElement()) );
+  actionCollection()->addAction( "xml_tool_insert_element", actionInsert );
+
+  KAction *actionClose = new KAction ( i18n("&Close Element"), this );
+  actionClose->setShortcut( Qt::CTRL+Qt::Key_Less );
+  connect( actionClose, SIGNAL(triggered()), &m_model, SLOT(slotCloseElement()) );
+  actionCollection()->addAction( "xml_tool_close_element", actionClose );
+
+  KAction *actionAssignDTD = new KAction ( i18n("Assign Meta &DTD..." ), this );
+  connect( actionAssignDTD, SIGNAL(triggered()), &m_model, SLOT(getDTD()) );
+  actionCollection()->addAction( "xml_tool_assign", actionAssignDTD );
+
+  setXMLFile( "plugins/katexmltools/ui.rc" );
+  win->guiFactory()->addClient( this );
+
+  connect( application()->documentManager(), SIGNAL(documentDeleted(KTextEditor::Document *)),
+            &m_model, SLOT(slotDocumentDeleted(KTextEditor::Document *)) );
+
+}
+
+PluginKateXMLToolsView::~PluginKateXMLToolsView()
+{
+  //kDebug() << "xml tools descructor 1...";
+  //TODO: unregister the model
+}
+
+PluginKateXMLToolsCompletionModel::PluginKateXMLToolsCompletionModel( QObject *parent ) : CodeCompletionModel2 (parent)
+{
   m_dtdString.clear();
   m_urlString.clear();
   m_docToAssignTo = 0L;
@@ -128,189 +161,95 @@ PluginKateXMLTools::PluginKateXMLTools( QObject* parent, const QStringList& )
   m_mode = none;
   m_correctPos = 0;
 
-  m_lastLine = 0;
-  m_lastCol = 0;
-  m_lastAllowed = QStringList();
-  m_popupOpenCol = -1;
+  m_allowed = QStringList();
 
-  m_dtds.setAutoDelete( true );
-
-  m_documentManager = ((Kate::Application*)parent)->documentManager();
-
-//   connect( m_documentManager, SIGNAL(documentCreated()),
-//             this, SLOT(slotDocumentCreated()) );
-  connect( m_documentManager, SIGNAL(documentDeleted(uint)),
-            this, SLOT(slotDocumentDeleted(uint)) );
+  setHasGroups( false );
 }
 
-PluginKateXMLTools::~PluginKateXMLTools()
+PluginKateXMLToolsCompletionModel::~PluginKateXMLToolsCompletionModel()
 {
-  //kDebug() << "xml tools descructor 1...";
+  qDeleteAll( m_dtds );
+  m_dtds.clear();
 }
 
-void PluginKateXMLTools::storeViewConfig(KConfig* config, Kate::MainWindow* win, const QString& groupPrefix)
-{
-  // TODO: FIXME: port to new Kate interfaces
-}
- 
-void PluginKateXMLTools::loadViewConfig(KConfig* config, Kate::MainWindow*win, const QString& groupPrefix)
-{
-  // TODO: FIXME: port to new Kate interfaces
-}
-
-void PluginKateXMLTools::storeGeneralConfig(KConfig* config, const QString& groupPrefix)
-{
-  // TODO: FIXME: port to new Kate interfaces
-}
-
-void PluginKateXMLTools::loadGeneralConfig(KConfig* config, const QString& groupPrefix)
-{
-  // TODO: FIXME: port to new Kate interfaces
-}
-
-void PluginKateXMLTools::addView( Kate::MainWindow *win )
-{
-  // TODO: doesn't this have to be deleted?
-  PluginView *view = new PluginView ();
-  ( void) new KAction ( i18n("&Insert Element..."), Qt::CTRL+Qt::Key_Return, this,
-                        SLOT( slotInsertElement()), view->actionCollection(), "xml_tool_insert_element" );
-  ( void) new KAction ( i18n("&Close Element"), Qt::CTRL+Qt::Key_Less, this,
-                        SLOT( slotCloseElement()), view->actionCollection(), "xml_tool_close_element" );
-  ( void) new KAction ( i18n("Assign Meta &DTD..." ), 0, this,
-                        SLOT( getDTD()), view->actionCollection(), "xml_tool_assign" );
-
-  view->setComponentData( KComponentData("kate") );
-  view->setXMLFile( "plugins/katexmltools/ui.rc" );
-  win->guiFactory()->addClient( view );
-
-  view->win = win;
-  m_views.append( view );
-}
-
-void PluginKateXMLTools::removeView( Kate::MainWindow *win )
-{
-  for ( uint z=0; z < m_views.count(); z++ )
-  {
-    if ( m_views.at(z)->win == win )
-    {
-      PluginView *view = m_views.at( z );
-      m_views.remove ( view );
-      win->guiFactory()->removeClient( view );
-      delete view;
-    }
-  }
-}
-
-void PluginKateXMLTools::slotDocumentDeleted( uint documentNumber )
+void PluginKateXMLToolsCompletionModel::slotDocumentDeleted( KTextEditor::Document *doc )
 {
   // Remove the document from m_DTDs, and also delete the PseudoDTD
   // if it becomes unused.
-  if ( m_docDtds[ documentNumber ] )
+  if ( m_docDtds.contains (doc) )
   {
     kDebug()<<"XMLTools:slotDocumentDeleted: documents: "<<m_docDtds.count()<<", DTDs: "<<m_dtds.count();
-    PseudoDTD *dtd = m_docDtds.take( documentNumber );
+    PseudoDTD *dtd = m_docDtds.take( doc );
 
-    Q3IntDictIterator<PseudoDTD> it ( m_docDtds );
-    for ( ; it.current(); ++it )
-    {
-      if ( it.current() == dtd )
-        return;
-    }
+    if ( m_docDtds.key( dtd ) )
+      return;
 
-    Q3DictIterator<PseudoDTD> it1( m_dtds );
-    for ( ; it1.current() ; ++it1 )
+    QHash<QString, PseudoDTD *>::iterator it;
+    for ( it = m_dtds.begin() ; it != m_dtds.end() ; ++it )
     {
-      if ( it1.current() == dtd )
+      if ( it.value() == dtd )
       {
-        m_dtds.remove( it1.currentKey() );
+        m_dtds.erase(it);
+        delete dtd;
         return;
       }
     }
   }
 }
 
-void PluginKateXMLTools::backspacePressed()
+
+void PluginKateXMLToolsCompletionModel::completionInvoked( KTextEditor::View *kv,
+                                                           const KTextEditor::Range &range, InvocationType invocationType )
 {
-  kDebug() << "xml tools backspacePressed";
+  Q_UNUSED( range )
+  Q_UNUSED( invocationType )
 
-  if ( !application()->activeMainWindow() )
-    return;
+  kDebug() << "xml tools completionInvoked";
 
-  KTextEditor::View *kv = application()->activeMainWindow()->activeView();
-  if( ! kv )
-  {
-    kDebug() << "Warning: no KTextEditor::View";
-    return;
-  }
-  int line, col;
-  kv->cursorPosition().position ( line, col );
-
-  //kDebug() << "++ redisplay popup? line:" << line << ", col: " << col;
-  if( m_lastLine == line && col == m_lastCol )
-  {
-    int len = col - m_popupOpenCol;
-    if( len < 0 )
-    {
-      kDebug() << "**Warning: len < 0";
-      return;
-    }
-    //kDebug() << "++ redisplay popup, " << m_lastAllowed.count() << ", len:" << len;
-    connectSlots( kv );
-    kv->showCompletionBox( stringListToCompletionEntryList(m_lastAllowed), len, false );
-  }
-}
-
-void PluginKateXMLTools::emptyKeyEvent()
-{
-  keyEvent( 0, 0, QString() );
-}
-
-void PluginKateXMLTools::keyEvent( int, int, const QString &/*s*/ )
-{
-  //kDebug() << "xml tools keyEvent: '" << s;
-
-  if ( !application()->activeMainWindow() )
-    return;
-
-  KTextEditor::View *kv = application()->activeMainWindow()->activeView();
-  if( ! kv )
-  {
-    kDebug() << "Warning: no KTextEditor::View";
-    return;
-  }
-
-  uint docNumber = kv->document()->documentNumber();
-  if( ! m_docDtds[ docNumber ] )
+  KTextEditor::Document *doc = kv->document();
+  if( ! m_docDtds[ doc ] )
     // no meta DTD assigned yet
     return;
 
   // debug to test speed:
   //QTime t; t.start();
 
-  QStringList allowed = QStringList();
+  beginResetModel();
+  m_allowed.clear();
 
   // get char on the left of the cursor:
-  uint line, col;
-  kv->cursorPositionReal( &line, &col );
-  QString lineStr = kv->getDoc()->textLine( line );
+  KTextEditor::Cursor curpos = kv->cursorPosition();
+  uint line = curpos.line(), col = curpos.column();
+
+  QString lineStr = kv->document()->line( line );
   QString leftCh = lineStr.mid( col-1, 1 );
   QString secondLeftCh = lineStr.mid( col-2, 1 );
 
   if( leftCh == "&" )
   {
     kDebug() << "Getting entities";
-    allowed = m_docDtds[docNumber]->entities("" );
+    m_allowed = m_docDtds[doc]->entities("" );
     m_mode = entities;
   }
   else if( leftCh == "<" )
   {
     kDebug() << "*outside tag -> get elements";
-    QString parentElement = getParentElement( *kv, true );
+    QString parentElement = getParentElement( *kv, 1 );
     kDebug() << "parent: " << parentElement;
-    allowed = m_docDtds[docNumber]->allowedElements(parentElement );
+    m_allowed = m_docDtds[doc]->allowedElements(parentElement );
     m_mode = elements;
   }
-  // TODO: optionally close parent tag if not left=="/>"
+  else if ( leftCh == "/" && secondLeftCh == "<" )
+  {
+    kDebug() << "*close parent element";
+    QString parentElement = getParentElement( *kv, 2 );
+
+    if ( ! parentElement.isEmpty() )
+    {
+      m_mode = closingtag;
+      m_allowed = QStringList( parentElement );
+    }
+  }
   else if( leftCh == " " || (isQuote(leftCh) && secondLeftCh == "=") )
   {
     // TODO: check secondLeftChar, too?! then you don't need to trigger
@@ -326,14 +265,14 @@ void PluginKateXMLTools::keyEvent( int, int, const QString &/*s*/ )
     if( ! currentElement.isEmpty() && ! currentAttribute.isEmpty() )
     {
       kDebug() << "*inside attribute -> get attribute values";
-      allowed = m_docDtds[docNumber]->attributeValues(currentElement, currentAttribute );
-      if( allowed.count() == 1 &&
-          (allowed[0] == "CDATA" || allowed[0] == "ID" || allowed[0] == "IDREF" ||
-          allowed[0] == "IDREFS" || allowed[0] == "ENTITY" || allowed[0] == "ENTITIES" ||
-          allowed[0] == "NMTOKEN" || allowed[0] == "NMTOKENS" || allowed[0] == "NAME") )
+      m_allowed = m_docDtds[doc]->attributeValues(currentElement, currentAttribute );
+      if( m_allowed.count() == 1 &&
+          (m_allowed[0] == "CDATA" || m_allowed[0] == "ID" || m_allowed[0] == "IDREF" ||
+          m_allowed[0] == "IDREFS" || m_allowed[0] == "ENTITY" || m_allowed[0] == "ENTITIES" ||
+          m_allowed[0] == "NMTOKEN" || m_allowed[0] == "NMTOKENS" || m_allowed[0] == "NAME") )
       {
         // these must not be taken literally, e.g. don't insert the string "CDATA"
-        allowed.clear();
+        m_allowed.clear();
       }
       else
       {
@@ -343,70 +282,51 @@ void PluginKateXMLTools::keyEvent( int, int, const QString &/*s*/ )
     else if( ! currentElement.isEmpty() )
     {
       kDebug() << "*inside tag -> get attributes";
-      allowed = m_docDtds[docNumber]->allowedAttributes(currentElement );
+      m_allowed = m_docDtds[doc]->allowedAttributes(currentElement );
       m_mode = attributes;
     }
   }
 
   //kDebug() << "time elapsed (ms): " << t.elapsed();
-  //kDebug() << "Allowed strings: " << allowed.count();
+  kDebug() << "Allowed strings: " << m_allowed.count();
 
-  if( allowed.count() >= 1 && allowed[0] != "__EMPTY" )
+  if( m_allowed.count() >= 1 && m_allowed[0] != "__EMPTY" )
   {
-    allowed = sortQStringList( allowed );
-    connectSlots( kv );
-    kv->showCompletionBox( stringListToCompletionEntryList( allowed ), 0, false );
-    m_popupOpenCol = col;
-    m_lastAllowed = allowed;
+    m_allowed = sortQStringList( m_allowed );
   }
-  //else {
-  //  m_lastAllowed.clear();
-  //}
+  setRowCount( m_allowed.count() );
+  endResetModel();
 }
 
-Q3ValueList<KTextEditor::CompletionItem>
-PluginKateXMLTools::stringListToCompletionEntryList( QStringList list )
+QVariant PluginKateXMLToolsCompletionModel::data(const QModelIndex &idx, int role) const
 {
-  Q3ValueList<KTextEditor::CompletionItem> compList;
-  KTextEditor::CompletionItem entry;
-  for( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
-  {
-    entry=KTextEditor::CompletionItem(*it);
-    compList << entry;
-  }
-  return compList;
+  //kDebug() << "XMLToolsCompletionModel::data index " << idx << " role " << role;
+  if( (idx.column() == KTextEditor::CodeCompletionModel::Name) && (role == Qt::DisplayRole) )
+    return m_allowed.at( idx.row() );
+  else
+    return QVariant();
 }
 
 
-/**
- * disconnect all signals of a specified kateview from the local slots
- *
- */
-void  PluginKateXMLTools::disconnectSlots( KTextEditor::View *kv )
+bool PluginKateXMLToolsCompletionModel::shouldStartCompletion( KTextEditor::View *view,
+                                                               const QString &insertedText,
+                                                               bool userInsertion,
+                                                               const KTextEditor::Cursor &position )
 {
-  disconnect( kv, SIGNAL(filterInsertString(KTextEditor::CompletionItem*,QString*)), this, 0 );
-  disconnect( kv, SIGNAL(completionDone(KTextEditor::CompletionItem)), this, 0 );
-  disconnect( kv, SIGNAL(completionAborted()), this, 0 );
+  Q_UNUSED( view )
+  Q_UNUSED( userInsertion )
+  Q_UNUSED( position )
+  const QString triggerChars = "&</ '\""; // these are subsequently handled by completionInvoked()
+
+  return triggerChars.contains( insertedText.right(1) );
 }
 
-/**
- * connect all signals of a specified kateview to the local slots
- *
- */
-void PluginKateXMLTools::connectSlots( KTextEditor::View *kv )
-{
-  connect( kv, SIGNAL(filterInsertString(KTextEditor::CompletionItem*,QString*) ),
-          this, SLOT(filterInsertString(KTextEditor::CompletionItem*,QString*)) );
-  connect( kv, SIGNAL(completionDone(KTextEditor::CompletionItem) ),
-          this, SLOT(completionDone(KTextEditor::CompletionItem)) );
-  connect( kv, SIGNAL(completionAborted()), this, SLOT(completionAborted()) );
-}
 
 /**
  * Load the meta DTD. In case of success set the 'ready'
  * flag to true, to show that we're is ready to give hints about the DTD.
  */
-void PluginKateXMLTools::getDTD()
+void PluginKateXMLToolsCompletionModel::getDTD()
 {
   if ( !application()->activeMainWindow() )
     return;
@@ -434,10 +354,10 @@ void PluginKateXMLTools::getDTD()
   // Example syntax:
   // <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "DTD/xhtml1-transitional.dtd">
   uint checkMaxLines = 200;
-  QString documentStart = kv->getDoc()->text(0, 0, checkMaxLines+1, 0 );
-  QRegExp re( "<!DOCTYPE\\s+(.*)\\s+PUBLIC\\s+[\"'](.*)[\"']", false );
+  QString documentStart = kv->document()->text( KTextEditor::Range(0, 0, checkMaxLines+1, 0) );
+  QRegExp re( "<!DOCTYPE\\s+(.*)\\s+PUBLIC\\s+[\"'](.*)[\"']", Qt::CaseInsensitive );
   re.setMinimal( true );
-  int matchPos = re.search( documentStart );
+  int matchPos = re.indexIn( documentStart );
   QString filename;
   QString doctype;
   QString topElement;
@@ -463,8 +383,8 @@ void PluginKateXMLTools::getDTD()
     else if ( doctype == "-//KDE//DTD DocBook XML V4.1.2-Based Variant V1.1//EN" )
       filename = "kde-docbook.dtd.xml";
   }
-  else if( documentStart.find("<xsl:stylesheet" ) != -1 &&
-             documentStart.find( "xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\"") != -1 )
+  else if( documentStart.indexOf("<xsl:stylesheet" ) != -1 &&
+             documentStart.indexOf( "xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\"") != -1 )
   {
     /* XSLT doesn't have a doctype/DTD. We look for an xsl:stylesheet tag instead.
       Example:
@@ -506,7 +426,7 @@ void PluginKateXMLTools::getDTD()
     m_dtdString = "";
     m_docToAssignTo = kv->document();
 
-    QApplication::setOverrideCursor( KCursor::waitCursor() );
+    KApplication::setOverrideCursor( Qt::WaitCursor );
     KIO::Job *job = KIO::get( url );
     connect( job, SIGNAL(result(KJob *)), this, SLOT(slotFinished(KJob *)) );
     connect( job, SIGNAL(data(KIO::Job *, const QByteArray &)),
@@ -515,12 +435,12 @@ void PluginKateXMLTools::getDTD()
   kDebug()<<"XMLTools::getDTD: Documents: "<<m_docDtds.count()<<", DTDs: "<<m_dtds.count();
 }
 
-void PluginKateXMLTools::slotFinished( KJob *job )
+void PluginKateXMLToolsCompletionModel::slotFinished( KJob *job )
 {
   if( job->error() )
   {
     //kDebug() << "XML Plugin error: DTD in XML format (" << filename << " ) could not be loaded";
-    static_cast<KIO::Job*>(job)->showErrorDialog( 0 );
+    static_cast<KIO::Job*>(job)->ui()->showErrorMessage();
   }
   else if ( static_cast<KIO::TransferJob *>(job)->isErrorPage() )
   {
@@ -544,20 +464,25 @@ void PluginKateXMLTools::slotFinished( KJob *job )
   QApplication::restoreOverrideCursor();
 }
 
-void PluginKateXMLTools::slotData( KIO::Job *, const QByteArray &data )
+void PluginKateXMLToolsCompletionModel::slotData( KIO::Job *, const QByteArray &data )
 {
   m_dtdString += QString( data );
 }
 
-void PluginKateXMLTools::assignDTD( PseudoDTD *dtd, KTextEditor::Document *doc )
+void PluginKateXMLToolsCompletionModel::assignDTD( PseudoDTD *dtd, KTextEditor::Document *doc )
 {
-  m_docDtds.replace( doc->documentNumber(), dtd );
-  connect( doc, SIGNAL(charactersInteractivelyInserted(int,int,const QString&) ),
-           this, SLOT(keyEvent(int,int,const QString&)) );
+  m_docDtds.insert( doc, dtd );
 
-  disconnect( doc, SIGNAL(backspacePressed()), this, 0 );
-  connect( doc, SIGNAL(backspacePressed() ),
-           this, SLOT(backspacePressed()) );
+  KTextEditor::CodeCompletionInterface *cci = qobject_cast<KTextEditor::CodeCompletionInterface *>(doc->activeView()); //TODO:perhaps foreach views()?
+
+  if( cci ) {
+    cci->registerCompletionModel( this );
+    cci->setAutomaticInvocationEnabled( true );
+    kDebug() << "PluginKateXMLToolsView: completion model registered";
+  }
+  else {
+    kWarning() << "PluginKateXMLToolsView: completion interface unavailable";
+  }
 }
 
 /**
@@ -565,7 +490,7 @@ void PluginKateXMLTools::assignDTD( PseudoDTD *dtd, KTextEditor::Document *doc )
  * tag one chosen/entered by the user, plus its closing tag. If there's a text selection,
  * add the markup around it.
  */
-void PluginKateXMLTools::slotInsertElement()
+void PluginKateXMLToolsCompletionModel::slotInsertElement()
 {
   if ( !application()->activeMainWindow() )
     return;
@@ -577,8 +502,8 @@ void PluginKateXMLTools::slotInsertElement()
     return;
   }
 
-  PseudoDTD *dtd = m_docDtds[kv->document()->documentNumber()];
-  QString parentElement = getParentElement( *kv, false );
+  PseudoDTD *dtd = m_docDtds[kv->document()];
+  QString parentElement = getParentElement( *kv, 0 );
   QStringList allowed;
 
   if( dtd )
@@ -591,7 +516,7 @@ void PluginKateXMLTools::slotInsertElement()
 
   if( !text.isEmpty() )
   {
-    QStringList list = QStringList::split( ' ', text );
+    QStringList list = text.split( QChar(' ') );
     QString pre;
     QString post;
     // anders: use <tagname/> if the tag is required to be empty.
@@ -616,10 +541,10 @@ void PluginKateXMLTools::slotInsertElement()
 
     QString marked;
     if ( ! post.isEmpty() )
-      marked = kv->getDoc()->selection();
+      marked = kv->selectionText();
 
     if( marked.length() > 0 )
-      kv->getDoc()->removeSelectedText();
+      kv->removeSelectionText();
 
     kv->insertText( pre + marked + post );
   }
@@ -628,7 +553,7 @@ void PluginKateXMLTools::slotInsertElement()
 /**
  * Insert a closing tag for the nearest not-closed parent element.
  */
-void PluginKateXMLTools::slotCloseElement()
+void PluginKateXMLToolsCompletionModel::slotCloseElement()
 {
   if ( !application()->activeMainWindow() )
     return;
@@ -639,7 +564,7 @@ void PluginKateXMLTools::slotCloseElement()
     kDebug() << "Warning: no KTextEditor::View";
     return;
   }
-  QString parentElement = getParentElement( *kv, false );
+  QString parentElement = getParentElement( *kv, 0 );
 
   //kDebug() << "parentElement: '" << parentElement << "'";
   QString closeTag = "</" + parentElement + '>';
@@ -647,47 +572,46 @@ void PluginKateXMLTools::slotCloseElement()
     kv->insertText( closeTag );
 }
 
+
 // modify the completion string before it gets inserted
-void PluginKateXMLTools::filterInsertString( KTextEditor::CompletionItem *ce, QString *text )
+void PluginKateXMLToolsCompletionModel::executeCompletionItem2( KTextEditor::Document *document,
+                                                                const KTextEditor::Range &word,
+                                                                const QModelIndex &index ) const
 {
-  kDebug() << "filterInsertString str: " << *text;
-  kDebug() << "filterInsertString text: " << ce.text();
+  KTextEditor::Range toReplace = word;
 
-  if ( !application()->activeMainWindow() )
-    return;
+  QString text = data( index.sibling( index.row(), Name ), Qt::DisplayRole ).toString();
 
-  KTextEditor::View *kv = application()->activeMainWindow()->activeView();
+  kDebug() << "executeCompletionItem text: " << text;
+
+  KTextEditor::View *kv = document->activeView();
   if( ! kv )
   {
     kDebug() << "Warning (filterInsertString() ): no KTextEditor::View";
     return;
   }
 
-  uint line, col;
-  kv->cursorPositionReal( &line, &col );
-  QString lineStr = kv->getDoc()->textLine(line );
+  int line, col;
+  kv->cursorPosition().position( line, col );
+  QString lineStr = document->line( line );
   QString leftCh = lineStr.mid( col-1, 1 );
   QString rightCh = lineStr.mid( col, 1 );
 
-  m_correctPos = 0;	// where to move the cursor after completion ( >0 = move right )
+  int posCorrection = 0;	// where to move the cursor after completion ( >0 = move right )
   if( m_mode == entities )
   {
-    // This is a bit ugly, but entities are case-sensitive
-    // and we want the correct completion even if the user started typing
-    // e.g. in lower case but the entity is in upper case
-    kv->getDoc()->removeText( line, col - (ce->text.length() - text->length()), line, col );
-    *text = ce->text + ';';
+    text = text + ';';
   }
 
   else if( m_mode == attributes )
   {
-    *text = *text + "=\"\"";
-    m_correctPos = -1;
+    text = text + "=\"\"";
+    posCorrection = -1;
     if( !rightCh.isEmpty() && rightCh != ">" && rightCh != "/" && rightCh != " " )
     {	// TODO: other whitespaces
       // add space in front of the next attribute
-      *text = *text + ' ';
-      m_correctPos--;
+      text = text + ' ';
+      posCorrection--;
     }
   }
 
@@ -706,109 +630,53 @@ void PluginKateXMLTools::filterInsertString( KTextEditor::CompletionItem *ce, QS
     }
 
     // find right quote:
-    for( endAttValue = col; endAttValue <= lineStr.length(); endAttValue++ )
+    for( endAttValue = col; endAttValue <= (uint) lineStr.length(); endAttValue++ )
     {
       QString ch = lineStr.mid( endAttValue-1, 1 );
       if( isQuote(ch) )
         break;
     }
 
-    // maybe the user has already typed something to trigger completion,
-    // don't overwrite that:
-    startAttValue += ce->text.length() - text->length();
-		// delete the current contents of the attribute:
+    // replace the current contents of the attribute
     if( startAttValue < endAttValue )
-    {
-      kv->getDoc()->removeText( line, startAttValue, line, endAttValue-1 );
-      // FIXME: this makes the scrollbar jump
-      // but without it, inserting sometimes goes crazy :-(
-      kv->setCursorPositionReal( line, startAttValue );
-    }
+      toReplace = KTextEditor::Range( line, startAttValue, line, endAttValue-1 );
   }
 
   else if( m_mode == elements )
   {
     // anders: if the tag is marked EMPTY, insert in form <tagname/>
     QString str;
-    int docNumber = kv->document()->documentNumber();
-    bool isEmptyTag =m_docDtds[docNumber]->allowedElements(ce->text).contains( "__EMPTY" );
+    bool isEmptyTag =m_docDtds[document]->allowedElements(text).contains( "__EMPTY" );
     if ( isEmptyTag )
-      str = "/>";
+      str = text + "/>";
     else
-      str = "></" + ce->text + '>';
-    *text = *text + str;
+      str = text + "></" + text + '>';
 
     // Place the cursor where it is most likely wanted:
     // always inside the tag if the tag is empty AND the DTD indicates that there are attribs)
     // outside for open tags, UNLESS there are mandatory attributes
-    if ( m_docDtds[docNumber]->requiredAttributes(ce->text).count()
-         || ( isEmptyTag && m_docDtds[docNumber]->allowedAttributes( ce->text).count() ) )
-      m_correctPos = - str.length();
+    if ( m_docDtds[document]->requiredAttributes(text).count()
+         || ( isEmptyTag && m_docDtds[document]->allowedAttributes(text).count() ) )
+      posCorrection = text.length() - str.length();
     else if ( ! isEmptyTag )
-      m_correctPos = -str.length() + 1;
+      posCorrection = text.length() - str.length() + 1;
+
+    text = str;
   }
+
+  else if( m_mode == closingtag )
+  {
+    text += ">";
+  }
+
+  document->replaceText( toReplace, text );
+
+  // move the cursor to desired position
+  KTextEditor::Cursor curPos = kv->cursorPosition();
+  curPos.setColumn( curPos.column() + posCorrection );
+  kv->setCursorPosition( curPos );
 }
 
-static void correctPos( KTextEditor::View *kv, int count )
-{
-  if( count > 0 )
-  {
-    for( int i = 0; i < count; i++ )
-      kv->cursorRight();
-  }
-  else if( count < 0 )
-  {
-    for( int i = 0; i < -count; i++ )
-      kv->cursorLeft();
-  }
-}
-
-void PluginKateXMLTools::completionAborted()
-{
-  if ( !application()->activeMainWindow() )
-    return;
-
-  KTextEditor::View *kv = application()->activeMainWindow()->activeView();
-  if( ! kv )
-  {
-    kDebug() << "Warning (completionAborted() ): no KTextEditor::View";
-    return;
-  }
-  disconnectSlots( kv );
-  kv->cursorPosition().position ( m_lastLine, m_lastCol );
-  m_lastCol--;
-
-  correctPos( kv,m_correctPos );
-  m_correctPos = 0;
-
-  kDebug() << "completionAborted() at line:" << m_lastLine << ", col:" << m_lastCol;
-}
-
-void PluginKateXMLTools::completionDone( KTextEditor::CompletionItem )
-{
-  kDebug() << "completionDone()";
-
-  if ( !application()->activeMainWindow() )
-    return;
-
-  KTextEditor::View *kv = application()->activeMainWindow()->activeView();
-  if( ! kv )
-  {
-    kDebug() << "Warning (completionDone() ): no KTextEditor::View";
-    return;
-  }
-  disconnectSlots( kv );
-
-  correctPos( kv,m_correctPos );
-  m_correctPos = 0;
-
-  if( m_mode == attributes )
-  {
-    // immediately show attribute values:
-    QTimer::singleShot( 10, this, SLOT(emptyKeyEvent()) );
-  }
-
-}
 
 // ========================================================================
 // Pseudo-XML stuff:
@@ -818,7 +686,7 @@ void PluginKateXMLTools::completionDone( KTextEditor::CompletionItem )
  * if "<" occurs before ">" occurs ( on the left side of the cursor ).
  * Return the tag name, return "" if we cursor is outside a tag.
  */
-QString PluginKateXMLTools::insideTag( KTextEditor::View &kv )
+QString PluginKateXMLToolsCompletionModel::insideTag( KTextEditor::View &kv )
 {
   int line, col;
   kv.cursorPosition().position( line, col );
@@ -869,7 +737,7 @@ QString PluginKateXMLTools::insideTag( KTextEditor::View &kv )
  * Note: only call when insideTag() == true.
  * TODO: allow whitespace around "="
  */
-QString PluginKateXMLTools::insideAttribute( KTextEditor::View &kv )
+QString PluginKateXMLToolsCompletionModel::insideAttribute( KTextEditor::View &kv )
 {
   int line, col;
   kv.cursorPosition().position( line, col );
@@ -927,7 +795,7 @@ QString PluginKateXMLTools::insideAttribute( KTextEditor::View &kv )
  * <p> foo <img/> bar X
  * <p> foo bar X
  */
-QString PluginKateXMLTools::getParentElement( KTextEditor::View &kv, bool ignoreSingleChar )
+QString PluginKateXMLToolsCompletionModel::getParentElement( KTextEditor::View &kv, int skipCharacters )
 {
   enum {
     parsingText,
@@ -938,7 +806,7 @@ QString PluginKateXMLTools::getParentElement( KTextEditor::View &kv, bool ignore
     parsingAttributeSquote,
     parsingIgnore
   } parseState;
-  parseState = ignoreSingleChar ? parsingIgnore : parsingText;
+  parseState = (skipCharacters > 0) ? parsingIgnore : parsingText;
 
   int nestingLevel = 0;
 
@@ -965,7 +833,8 @@ QString PluginKateXMLTools::getParentElement( KTextEditor::View &kv, bool ignore
     switch( parseState )
     {
       case parsingIgnore:
-        parseState = parsingText;
+        // ignore the specified number of characters
+        parseState = ( --skipCharacters > 0 ) ? parsingIgnore : parsingText;
         break;
 
       case parsingText:
@@ -1059,7 +928,7 @@ QString PluginKateXMLTools::getParentElement( KTextEditor::View &kv, bool ignore
  * Return true if the tag is neither a closing tag
  * nor an empty tag, nor a comment, nor processing instruction.
  */
-bool PluginKateXMLTools::isOpeningTag( QString tag )
+bool PluginKateXMLToolsCompletionModel::isOpeningTag( QString tag )
 {
   return ( !isClosingTag(tag) && !isEmptyTag(tag ) &&
       !tag.startsWith( "<?") && !tag.startsWith("<!") );
@@ -1069,12 +938,12 @@ bool PluginKateXMLTools::isOpeningTag( QString tag )
  * Return true if the tag is a closing tag. Return false
  * if the tag is an opening tag or an empty tag ( ! )
  */
-bool PluginKateXMLTools::isClosingTag( QString tag )
+bool PluginKateXMLToolsCompletionModel::isClosingTag( QString tag )
 {
   return ( tag.startsWith("</") );
 }
 
-bool PluginKateXMLTools::isEmptyTag( QString tag )
+bool PluginKateXMLToolsCompletionModel::isEmptyTag( QString tag )
 {
   return ( tag.right(2) == "/>" );
 }
@@ -1082,7 +951,7 @@ bool PluginKateXMLTools::isEmptyTag( QString tag )
 /**
  * Return true if ch is a single or double quote. Expects ch to be of length 1.
  */
-bool PluginKateXMLTools::isQuote( QString ch )
+bool PluginKateXMLToolsCompletionModel::isQuote( QString ch )
 {
   return ( ch == "\"" || ch == "'" );
 }
@@ -1092,23 +961,23 @@ bool PluginKateXMLTools::isQuote( QString ch )
 // Tools:
 
 /** Sort a QStringList case-insensitively. Static. TODO: make it more simple. */
-QStringList PluginKateXMLTools::sortQStringList( QStringList list ) {
+QStringList PluginKateXMLToolsCompletionModel::sortQStringList( QStringList list ) {
   // Sort list case-insensitive. This looks complicated but using a QMap
   // is even suggested by the Qt documentation.
   QMap<QString,QString> mapList;
   for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
   {
     QString str = *it;
-    if( mapList.contains(str.lower()) )
+    if( mapList.contains(str.toLower()) )
     {
       // do not override a previous value, e.g. "Auml" and "auml" are two different
       // entities, but they should be sorted next to each other.
       // TODO: currently it's undefined if e.g. "A" or "a" comes first, it depends on
       // the meta DTD ( really? it seems to work okay?!? )
-      mapList[str.lower()+'_'] = str;
+      mapList[str.toLower()+'_'] = str;
     }
     else
-      mapList[str.lower()] = str;
+      mapList[str.toLower()] = str;
   }
 
   list.clear();
@@ -1116,7 +985,7 @@ QStringList PluginKateXMLTools::sortQStringList( QStringList list ) {
 
   // Qt doc: "the items are alphabetically sorted [by key] when iterating over the map":
   for( it = mapList.begin(); it != mapList.end(); ++it )
-    list.append( it.data() );
+    list.append( it.value() );
 
   return list;
 }
@@ -1125,6 +994,8 @@ QStringList PluginKateXMLTools::sortQStringList( QStringList list ) {
 InsertElement::InsertElement( QWidget *parent, const char *name )
   :KDialog( parent)
 {
+  Q_UNUSED( name )
+
   setCaption(i18n("Insert XML Element" ));
   setButtons(KDialog::Ok|KDialog::Cancel);
   setDefaultButton(KDialog::Cancel);
@@ -1144,14 +1015,14 @@ QString InsertElement::showDialog( QStringList &completions )
 {
   QWidget *page = new QWidget( this );
   setMainWidget( page );
-  QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
+  QVBoxLayout *topLayout = new QVBoxLayout( page );
 
-  KHistoryCombo *combo = new KHistoryCombo( page );
+  KHistoryComboBox *combo = new KHistoryComboBox( page );
   combo->setHistoryItems( completions, true );
   connect( combo->lineEdit(), SIGNAL(textChanged ( const QString & )),
            this, SLOT(slotHistoryTextChanged(const QString &)) );
   QString text = i18n( "Enter XML tag name and attributes (\"<\", \">\" and closing tag will be supplied):" );
-  QLabel *label = new QLabel( text, page, "insert" );
+  QLabel *label = new QLabel( text, page );
 
   topLayout->addWidget( label );
   topLayout->addWidget( combo );
