@@ -146,6 +146,7 @@ public slots:
   void setRelativeCost();
   void toggleExpanded();
   void toggleCycles();
+  void toggleHideTemplates();
   void forceTrace();
   void forceTraceReload();
   void forwardAboutToShow();
@@ -220,6 +221,9 @@ private:
   void restoreTraceTypes();
   void restoreTraceSettings();
   void updateViewsOnChange(int);
+  /// open @p file, might be compressed
+  /// @return true when the file could be opened, false otherwise.
+  bool openDataFile(const QString& file);
 
   KStatusBar* _statusbar;
   QLabel* _statusLabel;
@@ -240,7 +244,7 @@ private:
   KSelectAction *_saCost, *_saCost2, *saGroup;
   KToggleAction *_partDockShown, *_stackDockShown;
   KToggleAction *_functionDockShown, *_dumpDockShown;
-  KToggleAction *_taPercentage, *_taExpanded, *_taCycles;
+  KToggleAction *_taPercentage, *_taExpanded, *_taCycles, *_taHideTemplates;
   KToggleAction *_taDump, *_taSplit, *_taSplitDir;
   KToolBarPopupAction *_paForward, *_paBack, *_paUp;
 
@@ -278,7 +282,7 @@ private:
   QProgressBar* _progressBar;
 
   // toplevel configuration options
-  bool _showPercentage, _showExpanded, _showCycles;
+  bool _showPercentage, _showExpanded, _showCycles, _hideTemplates;
 };
 
 #endif
