@@ -96,7 +96,7 @@ void KompareProcess::writeDefaultCommandLine()
 
 void KompareProcess::writeCommandLine()
 {
-	// load the executable into the K3Process
+	// load the executable into the KProcess
 	if ( m_diffSettings->m_diffProgram.isEmpty() )
 	{
 		kDebug(8101) << "Using the first diff in the path..." << endl;
@@ -173,7 +173,7 @@ void KompareProcess::writeCommandLine()
 
 	if ( m_diffSettings->m_ignoreRegExp && !m_diffSettings->m_ignoreRegExpText.isEmpty() )
 	{
-		*this << "-I " << m_diffSettings->m_ignoreRegExpText;
+		*this << "-I" << m_diffSettings->m_ignoreRegExpText;
 	}
 
 	if ( m_diffSettings->m_showCFunctionChange )
@@ -229,7 +229,7 @@ void KompareProcess::setEncoding( const QString& encoding )
 	}
 	else
 	{
-		m_codec = KGlobal::charsets()->codecForName( encoding.latin1() );
+		m_codec = KGlobal::charsets()->codecForName( encoding.toLatin1() );
 		if ( m_codec )
 			m_textDecoder = m_codec->makeDecoder();
 		else
