@@ -188,6 +188,8 @@ public slots: //DBus interface
     bool save();
     QByteArray contents();
     QString mimetype();
+    QString sourceLangCode() const;
+    QString targetLangCode() const;
 
 protected:
     virtual KAutoSaveFile* checkAutoSave(const KUrl& url);
@@ -253,6 +255,7 @@ signals:
     void signalFileLoaded(const KUrl&);
     Q_SCRIPTABLE void signalFileSaved();
     void signalFileSaved(const KUrl&);
+    void signalFileAutoSaveFailed(const QString&);
 };
 
 
