@@ -1,12 +1,11 @@
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *  copyright (C) 2002-2008                                                *
- *  Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                   *
+ *   copyright (C) 2002-2009                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
  /* Created By Bouchikhi Mohamed-Amine */
@@ -14,33 +13,26 @@
 #ifndef CMD_CHANGEMULTI_H
 #define CMD_CHANGEMULTI_H
 
-
 #include <QtGui/QUndoCommand>
 
-#include "umlwidget.h"
-#include "umlrole.h"
+class UMLRole;
 
 namespace Uml
 {
-    class CmdChangeMulti: public QUndoCommand
+    class CmdChangeMulti : public QUndoCommand
     {
-        /* Object selected */
-        UMLRole *UMLr;
-
-        /* Old multiplicity */
-        QString oldMulti;
-
-        /* New multiplicity */
-        QString newMulti;
-
-
     public:
-        CmdChangeMulti(UMLRole *role,const QString &multi);
+        CmdChangeMulti(UMLRole *role, const QString &multi);
         void redo();
         void undo();
+
+    private:
+        UMLRole* m_umlRole;   ///< Object selected
+        QString m_oldMulti;   ///< Old multiplicity
+        QString m_newMulti;   ///< New multiplicity
     };
 
 }
 
-#endif /*CMD_CHANGEMULTI_H*/
+#endif // CMD_CHANGEMULTI_H
 
