@@ -12,7 +12,7 @@
 #ifndef OVERWRITEDIALOGUE_H
 #define OVERWRITEDIALOGUE_H
 
-#include <kdialogbase.h>
+#include <kdialog.h>
 
 class QCheckBox;
 
@@ -27,42 +27,31 @@ class QCheckBox;
  * @author Jonathan Riddell <jr@jriddell.org>
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
-class OverwriteDialogue: public KDialogBase {
+class OverwriteDialogue: public KDialog {
     Q_OBJECT
 
 public:
+
     /**
      * constructor sets up the dialog, adding checkbox and label
      */
     OverwriteDialogue(const QString& fileName, const QString& outputDirectory,
-                      bool applyToAllRemaining, QWidget* parent=0, const char* name=0);
+                      bool applyToAllRemaining, QWidget* parent=0);
 
     /**
      * destrictor doesn't do anything
      */
     ~OverwriteDialogue();
 
-    /**
-     * @return the value of the Apply To All Remaining Files checkbox
-     */
     bool applyToAllRemaining();
 
 protected slots:
-    /**
-     * Overrides standard operation to call QDialog::done(Yes).
-     *  This is a kludge, see note in class description.
-     */
+
     virtual void slotOk();
 
-    /**
-    * Overrides standard operation to call QDialog::done(No).
-    */
     virtual void slotApply();
 
 
-    /**
-    * Overrides standard operation to call QDialog::done(Cancel).
-    */
     virtual void slotCancel();
 
 private:
