@@ -12,12 +12,12 @@
 #ifndef UMLSTEREOTYPELIST_H
 #define UMLSTEREOTYPELIST_H
 
-#include <qptrlist.h>
+#include <qlist.h>
 
 // forward declaration
 class UMLStereotype;
 
-typedef QPtrList<UMLStereotype> UMLStereotypeList;
-typedef QPtrListIterator<UMLStereotype> UMLStereotypeListIt;
+typedef QList<UMLStereotype*> UMLStereotypeList;
+typedef QListIterator<UMLStereotype*> UMLStereotypeListIt;
 
 #endif
